@@ -230,7 +230,12 @@ typedef struct {
 #define AG_ITER_STAGE_ACCOUNT   1
 #define AG_ITER_STAGE_SERVICE   2
 
-G_DEFINE_TYPE (AgAccount, ag_account, G_TYPE_OBJECT);
+static void ag_account_initable_iface_init(gpointer g_iface,
+                                           gpointer iface_data);
+
+G_DEFINE_TYPE_WITH_CODE (AgAccount, ag_account, G_TYPE_OBJECT,
+                         G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE,
+                                            ag_account_initable_iface_init));
 
 #define AG_ACCOUNT_PRIV(obj) (AG_ACCOUNT(obj)->priv)
 
@@ -273,8 +278,8 @@ ag_variant_safe_unref (gpointer variant)
 }
 
 GVariant *
-_ag_account_build_signal (AgAccount *account, AgAccountChanges *changes,
-                          const struct timespec *ts)
+_ag_account_build_dbus_changes (AgAccount *account, AgAccountChanges *changes,
+                                const struct timespec *ts)
 {
     GVariantBuilder builder;
     const gchar *provider_name;
@@ -284,8 +289,11 @@ _ag_account_build_signal (AgAccount *account, AgAccountChanges *changes,
     provider_name = account->priv->provider_name;
     if (!provider_name) provider_name = "";
 
-    g_variant_builder_add (&builder, "u", ts->tv_sec);
-    g_variant_builder_add (&builder, "u", ts->tv_nsec);
+    if (ts)
+    {
+        g_variant_builder_add (&builder, "u", ts->tv_sec);
+        g_variant_builder_add (&builder, "u", ts->tv_nsec);
+    }
     g_variant_builder_add (&builder, "u", account->id);
     g_variant_builder_add (&builder, "b", changes->created);
     g_variant_builder_add (&builder, "b", changes->deleted);
@@ -517,6 +525,9 @@ ag_service_changes_free (AgServiceChanges *sc)
 {
     g_free (sc->service_type);
 
+    if (sc->service)
+        ag_service_unref (sc->service);
+
     if (sc->settings)
         g_hash_table_unref (sc->settings);
 
@@ -604,51 +615,53 @@ update_settings (AgAccount *account, GHashTable *services)
             if (!priv->services) continue;
             ss = g_hash_table_lookup (priv->services, service_name);
         }
-        if (!ss) continue;
 
         /* get the watches associated to this service */
-        if (priv->watches)
+        if (ss != NULL && priv->watches != NULL)
             watches = g_hash_table_lookup (priv->watches, ss->service);
 
         g_hash_table_iter_init (&si, sc->settings);
         while (g_hash_table_iter_next (&si,
                                        (gpointer)&key, (gpointer)&value))
         {
-            if (ss->service == NULL)
+            if (ss != NULL)
             {
-                if (strcmp (key, "name") == 0)
+                if (ss->service == NULL)
                 {
-                    g_free (priv->display_name);
-                    priv->display_name =
-                        value ? g_variant_dup_string (value, NULL) : NULL;
-                    g_signal_emit (account, signals[DISPLAY_NAME_CHANGED], 0);
-                    g_object_notify_by_pspec ((GObject *)account,
-                                              properties[PROP_DISPLAY_NAME]);
-                    continue;
-                }
-                else if (strcmp (key, "enabled") == 0)
-                {
-                    priv->enabled =
-                        value ? g_variant_get_boolean (value) : FALSE;
-                    g_signal_emit (account, signals[ENABLED], 0,
-                                   NULL, priv->enabled);
-                    g_object_notify_by_pspec ((GObject *)account,
-                                              properties[PROP_ENABLED]);
-                    continue;
+                    if (strcmp (key, "name") == 0)
+                    {
+                        g_free (priv->display_name);
+                        priv->display_name =
+                            value ? g_variant_dup_string (value, NULL) : NULL;
+                        g_signal_emit (account, signals[DISPLAY_NAME_CHANGED], 0);
+                        g_object_notify_by_pspec ((GObject *)account,
+                                                  properties[PROP_DISPLAY_NAME]);
+                        continue;
+                    }
+                    else if (strcmp (key, "enabled") == 0)
+                    {
+                        priv->enabled =
+                            value ? g_variant_get_boolean (value) : FALSE;
+                        g_signal_emit (account, signals[ENABLED], 0,
+                                       NULL, priv->enabled);
+                        g_object_notify_by_pspec ((GObject *)account,
+                                                  properties[PROP_ENABLED]);
+                        continue;
+                    }
                 }
+
+                if (value)
+                    g_hash_table_replace (ss->settings,
+                                          g_strdup (key),
+                                          g_variant_ref (value));
+                else
+                    g_hash_table_remove (ss->settings, key);
+
+                /* check for installed watches to be invoked */
+                if (watches)
+                    watch_list = match_watch_with_key (watches, key, watch_list);
             }
 
-            if (value)
-                g_hash_table_replace (ss->settings,
-                                      g_strdup (key),
-                                      g_variant_ref (value));
-            else
-                g_hash_table_remove (ss->settings, key);
-
-            /* check for installed watches to be invoked */
-            if (watches)
-                watch_list = match_watch_with_key (watches, key, watch_list);
-
             if (strcmp (key, "enabled") == 0)
             {
                 gboolean enabled =
@@ -756,7 +769,7 @@ account_service_changes_get (AgAccountPrivate *priv, AgService *service,
     if (!sc)
     {
         sc = g_slice_new0 (AgServiceChanges);
-        sc->service = service;
+        sc->service = service ? ag_service_ref (service) : NULL;
         sc->service_type = g_strdup (service_type);
 
         sc->settings = g_hash_table_new_full
@@ -825,7 +838,7 @@ got_account (sqlite3_stmt *stmt, AgAccountPrivate *priv)
 }
 
 static gboolean
-ag_account_load (AgAccount *account)
+ag_account_load (AgAccount *account, GError **error)
 {
     AgAccountPrivate *priv = account->priv;
     gchar sql[128];
@@ -838,29 +851,24 @@ ag_account_load (AgAccount *account)
                                    (AgQueryCallback)got_account, priv, sql);
     /* if the query succeeded but we didn't get a row, we must set the
      * NOT_FOUND error */
-    if (_ag_manager_get_last_error (priv->manager) == NULL && rows != 1)
+    if (rows != 1)
     {
-        GError *error = g_error_new (AG_ACCOUNTS_ERROR,
-                                     AG_ACCOUNTS_ERROR_ACCOUNT_NOT_FOUND,
-                                     "Account %u not found in DB", account->id);
-        _ag_manager_take_error (priv->manager, error);
+        g_set_error (error,
+                     AG_ACCOUNTS_ERROR,
+                     AG_ACCOUNTS_ERROR_ACCOUNT_NOT_FOUND,
+                     "Account %u not found in DB", account->id);
     }
 
     return rows == 1;
 }
 
-static GObject *
-ag_account_constructor (GType type, guint n_params,
-                        GObjectConstructParam *params)
+static gboolean
+ag_account_initable_init (GInitable *initable,
+                          G_GNUC_UNUSED GCancellable *cancellable,
+                          GError **error)
 {
-    GObjectClass *object_class = (GObjectClass *)ag_account_parent_class;
-    GObject *object;
-    AgAccount *account;
+    AgAccount *account = AG_ACCOUNT (initable);
 
-    object = object_class->constructor (type, n_params, params);
-    g_return_val_if_fail (AG_IS_ACCOUNT (object), NULL);
-
-    account = AG_ACCOUNT (object);
     if (account->id)
     {
         if (account->priv->changes && account->priv->changes->created)
@@ -869,18 +877,25 @@ ag_account_constructor (GType type, guint n_params,
             _ag_account_changes_free (account->priv->changes);
             account->priv->changes = NULL;
         }
-        else if (!ag_account_load (account))
+        else if (!ag_account_load (account, error))
         {
             g_warning ("Unable to load account %u", account->id);
-            g_object_unref (object);
-            return NULL;
+            return FALSE;
         }
     }
 
     if (!account->priv->foreign)
         ag_account_select_service (account, NULL);
 
-    return object;
+    return TRUE;
+}
+
+static void
+ag_account_initable_iface_init (gpointer g_iface,
+                                G_GNUC_UNUSED gpointer iface_data)
+{
+    GInitableIface *iface = (GInitableIface *)g_iface;
+    iface->init = ag_account_initable_init;
 }
 
 static void
@@ -1004,7 +1019,6 @@ ag_account_class_init (AgAccountClass *klass)
 
     g_type_class_add_private (object_class, sizeof (AgAccountPrivate));
 
-    object_class->constructor = ag_account_constructor;
     object_class->get_property = ag_account_get_property;
     object_class->set_property = ag_account_set_property;
     object_class->dispose = ag_account_dispose;
@@ -1200,6 +1214,16 @@ _ag_account_changes_from_dbus (AgManager *manager, GVariant *v_services,
     return changes;
 }
 
+AgAccountChanges *
+_ag_account_steal_changes (AgAccount *account)
+{
+    AgAccountChanges *changes;
+
+    changes = account->priv->changes;
+    account->priv->changes = NULL;
+    return changes;
+}
+
 static void
 add_service_type (GPtrArray *types, const gchar *service_type)
 {
@@ -1328,8 +1352,8 @@ ag_account_store_signature (AgAccount *account, AgServiceChanges *sc, GString *s
     }
 }
 
-static gchar *
-ag_account_get_store_sql (AgAccount *account, GError **error)
+gchar *
+_ag_account_get_store_sql (AgAccount *account, GError **error)
 {
     AgAccountPrivate *priv;
     AgAccountChanges *changes;
@@ -2020,6 +2044,10 @@ ag_account_set_value (AgAccount *account, const gchar *key,
     }
 
     change_selected_service_value (priv, key, variant);
+    if (variant != NULL)
+    {
+        g_variant_unref (variant);
+    }
 }
 
 /**
@@ -2450,9 +2478,6 @@ ag_account_store_async (AgAccount *account, GCancellable *cancellable,
                         GAsyncReadyCallback callback, gpointer user_data)
 {
     AgAccountPrivate *priv;
-    AgAccountChanges *changes;
-    GError *error = NULL;
-    gchar *sql;
 
     g_return_if_fail (AG_IS_ACCOUNT (account));
     priv = account->priv;
@@ -2475,21 +2500,10 @@ ag_account_store_async (AgAccount *account, GCancellable *cancellable,
                                    ag_account_store_async);
     g_simple_async_result_set_check_cancellable (priv->store_async_result,
                                                  cancellable);
+    g_object_add_weak_pointer ((GObject *)priv->store_async_result,
+                               (gpointer *)&priv->store_async_result);
 
-    sql = ag_account_get_store_sql (account, &error);
-    if (G_UNLIKELY (error))
-    {
-        g_simple_async_result_take_error (priv->store_async_result,
-                                          error);
-        g_simple_async_result_complete_in_idle (priv->store_async_result);
-        g_clear_object (&priv->store_async_result);
-        return;
-    }
-
-    changes = priv->changes;
-    priv->changes = NULL;
-
-    if (G_UNLIKELY (!sql))
+    if (G_UNLIKELY (priv->changes == NULL))
     {
         /* Nothing to do: invoke the callback immediately */
         g_simple_async_result_complete_in_idle (priv->store_async_result);
@@ -2497,9 +2511,8 @@ ag_account_store_async (AgAccount *account, GCancellable *cancellable,
         return;
     }
 
-    _ag_manager_exec_transaction (priv->manager, sql, changes, account,
-                                  priv->store_async_result, cancellable);
-    g_free (sql);
+    _ag_manager_store_async (priv->manager, account,
+                             priv->store_async_result, cancellable);
 }
 
 /**
@@ -2541,44 +2554,17 @@ gboolean
 ag_account_store_blocking (AgAccount *account, GError **error)
 {
     AgAccountPrivate *priv;
-    AgAccountChanges *changes;
-    GError *error_int = NULL;
-    gchar *sql;
 
     g_return_val_if_fail (AG_IS_ACCOUNT (account), FALSE);
     priv = account->priv;
 
-    sql = ag_account_get_store_sql (account, &error_int);
-    if (G_UNLIKELY (error_int))
-    {
-        g_warning ("%s: %s", G_STRFUNC, error_int->message);
-        g_propagate_error (error, error_int);
-        return FALSE;
-    }
-
-    changes = priv->changes;
-    priv->changes = NULL;
-
-    if (G_UNLIKELY (!sql))
+    if (G_UNLIKELY (priv->changes == NULL))
     {
         /* Nothing to do: return immediately */
         return TRUE;
     }
 
-    _ag_manager_exec_transaction_blocking (priv->manager, sql,
-                                           changes, account,
-                                           &error_int);
-    g_free (sql);
-    _ag_account_changes_free (changes);
-
-    if (G_UNLIKELY (error_int))
-    {
-        g_warning ("%s: %s", G_STRFUNC, error_int->message);
-        g_propagate_error (error, error_int);
-        return FALSE;
-    }
-
-    return TRUE;
+    return _ag_manager_store_sync (priv->manager, account, error);
 }
 
 /**
